#include <iostream>
#include <vector>
#include <string>
#include <fstream>

#include "shops.h"
#include "equipment.h"
#include "characters.h"

using namespace std;


//CBlacksmith class
//constructor
CBlacksmith::CBlacksmith()
{
	AvailableWeapons.resize(NUM_OF_WEAPONS);
}
//CBlacksmith functions
//displays the items for sale at the blacksmith
void CBlacksmith::DisplayItemsForSale(CWorldEquipment WorldEquipment)
{
	cout<<"-------------------BLACKSMITH-------------------\n";
	//this is for display purposes, what number they have to enter to purchase the desired weapon
	int ShopIndex=1;

	//cycle through the available weapons at the dealer
	for(int index=0;index<NUM_OF_WEAPONS;index++)
	{
		//prevents the shop from showing any items that we don't want to show
		if(AvailableWeapons[index]>0 && AvailableWeapons[index]<NUM_OF_WEAPONS)
		{
			//for each weapon they want to have at the shop, display the weapon
			WorldEquipment.DisplayWeapon(ShopIndex,AvailableWeapons[index]);
			ShopIndex++;//increase the purchase #
		}
	}
	cout<<"Q)Exit Shop\n";
	cout<<"------------------------------------------------\n";
}
//returns a CWeapon that is the weapon purchased
bool CBlacksmith::PurchaseWeapon(CWorldEquipment WorldEquipment, CPlayerCharacter &Character,int PurchaseSlot)
{
	//the vector id 0 indexed so we have to subtract one from the choice to get the right item
	PurchaseSlot--;

	//this keeps the game from crashing, out of bounds array checking
	if(PurchaseSlot < 0 || PurchaseSlot > NUM_OF_WEAPONS)
	{
		cout<<"Invalid Choice\n";
		return false;
	}
	/*this looks like the same thing but it's not.  The above checks the bounds first to make sure
	this won't crash.  This checks to make sure that the slot they want to buy has a valid weapon ID.
	The vector will have more positions than it does values and we want to make sure we aren't checking for
	a value that hasn't been set yet.
	*/
	if(AvailableWeapons[PurchaseSlot]<0 || AvailableWeapons[PurchaseSlot] > NUM_OF_WEAPONS)
	{
		cout<<"Invalid Choice\n";
		return false;
	}

	for(int index=0;index<NUM_OF_WEAPONS;index++)
	{
		if(WorldEquipment.Weapons[index].ID==AvailableWeapons[PurchaseSlot])
		{
			if(Character.Gold<WorldEquipment.Weapons[index].PurchasePrice)
			{
				cout<<"Not enough gold\n";
				return false;
			}
			cout<<"You bought "<<WorldEquipment.Weapons[index].Name<<endl;

			Character.SetWeapon(WorldEquipment,WorldEquipment.Weapons[index].ID);
			Character.Gold-=WorldEquipment.Weapons[index].PurchasePrice;

			return true;
		}
	}

	/*if we made it this far then they chose something within the correct range of weapons but was not
	in the shop.  ie if the shop displayed 1-15 and num_of_weapons is 20 and they chose 16, it would
	not have passed the for loop above.  it could also be that the weapon id was bad, but that is a lot
	less likely then the case above and I'm not really sure how we'd check for both.*/
	cout<<"Invalid Choice\n";
	return false;
}


//CArmory class
//constructor
CArmory::CArmory()
{
	AvailableArmor.resize(NUM_OF_ARMOR);
}
//CArmory functions
//displays the items for sale at the Armory
void CArmory::DisplayItemsForSale(CWorldEquipment WorldEquipment)
{
	cout<<"--------------------ARMORY---------------------\n";
	//this is for display purposes, what number they have to enter to purchase the desired armor
	int ShopIndex=1;

	//cycle through the available armor at the dealer
	for(int index=0;index<NUM_OF_ARMOR;index++)
	{
		//prevents the shop from showing any items that we don't want to show
		if(AvailableArmor[index]>0 && AvailableArmor[index]<NUM_OF_ARMOR)
		{
			//for each armor they want to have at the shop, display the weapon
			WorldEquipment.DisplayArmor(ShopIndex,AvailableArmor[index]);
			ShopIndex++;//increase the purchase #
		}
	}
	cout<<"Q)Exit Shop\n";
	cout<<"------------------------------------------------\n";
}
//returns true if the purchase is successful
bool CArmory::PurchaseArmor(CWorldEquipment WorldEquipment, CPlayerCharacter &Character,int PurchaseSlot)
{
	//this keeps the game from crashing, out of bounds array checking
	if(PurchaseSlot < 0 || PurchaseSlot > NUM_OF_ARMOR)
	{
		cout<<"Invalid Choice\n";
		return false;
	}
	/*this looks like the same thing but it's not.  The above checks the bounds first to make sure
	this won't crash.  This checks to make sure that the slot they want to buy has a valid armor ID.
	The vector will have more positions than it does values and we want to make sure we aren't checking for
	a value that hasn't been set yet.
	*/
	if(AvailableArmor[PurchaseSlot]<0 || AvailableArmor[PurchaseSlot] > NUM_OF_ARMOR)
	{
		cout<<"Invalid Choice\n";
		return false;
	}

	for(int index=0;index<NUM_OF_ARMOR;index++)
	{
		//cycle through the world list until we find the correct armor ID
		if(WorldEquipment.Armor[index].ID==AvailableArmor[PurchaseSlot])
		{
			if(Character.Gold<WorldEquipment.Armor[index].PurchasePrice)
			{
				cout<<"Not enough gold\n";
				return false;
			}
			cout<<"You bought "<<WorldEquipment.Armor[index].Name<<endl;

			Character.SetArmor(WorldEquipment,WorldEquipment.Armor[index].ID);
			Character.Gold-=WorldEquipment.Armor[index].PurchasePrice;

			return true;
		}
	}

	/*if we made it this far then they chose something within the correct range of armor but was not
	in the shop.  ie if the shop displayed 1-15 and num_of_armor is 20 and they chose 16, it would
	not have passed the for loop above.  it could also be that the armor id was bad, but that is a lot
	less likely then the case above and I'm not really sure how we'd check for both.*/
	cout<<"Invalid Choice\n";
	return false;
}


//CShops class
//constructor
CShops::CShops()
{
	Blacksmiths.resize(NUM_OF_BLACKSMITHS);
	Armorers.resize(NUM_OF_ARMORERS);
}
//CShops functions
//loads the blacksmiths from a text file
void CShops::LoadBlacksmiths(char *filename)
{
	char TextInformation[1000];//holds input information from read files
	string TextParsed;//holds string token information

	//open file
	fstream file_open(filename,ios::in);
	//for each item in class array, get a new line
	while(!file_open.eof())
	{
		//get a new line from the file
		file_open.getline(TextInformation,1000);

		//set first item to the first semicolon
		TextParsed=strtok(TextInformation,";");

		//first item is blacksmith ID
		int ID=atoi(TextParsed.c_str() );
		Blacksmiths[ID].ID=ID;
		
		//get a new line for the available weapons
		file_open.getline(TextInformation,1000);

		//start an index for the available weapons, set it to 0
		int index=0;
		//first item is the first available weapon
		TextParsed=strtok(TextInformation,",");
		//set it to the first item
		Blacksmiths[ID].AvailableWeapons[index]=atoi(TextParsed.c_str() );
		
		//while we're not at the end of the available items...
		while(TextParsed.find(";")==string::npos)
		{
			index++;
			//keep adding new ones
			TextParsed=strtok(NULL,",");
			Blacksmiths[ID].AvailableWeapons[index]=atoi(TextParsed.c_str() );
		}

	}

	file_open.close();//close file

	cout<<"File closed\n";
}
//loads the armorers from a text file
void CShops::LoadArmorers(char *filename)
{
	char TextInformation[1000];//holds input information from read files
	string TextParsed;//holds string token information

	//open file
	fstream file_open(filename,ios::in);
	//for each item in class array, get a new line
	while(!file_open.eof())
	{
		//get a new line from the file
		file_open.getline(TextInformation,1000);

		//set first item to the first semicolon
		TextParsed=strtok(TextInformation,";");

		//first item is ID
		int ID=atoi(TextParsed.c_str() );
		Armorers[ID].ID=ID;
		
		//get a new line for the available armor
		file_open.getline(TextInformation,1000);

		//start an index for the available armor, set it to 0
		int index=0;
		//first item is the first available armor
		TextParsed=strtok(TextInformation,",");
		//set it to the first item
		Armorers[ID].AvailableArmor[index]=atoi(TextParsed.c_str() );
		
		//while we're not at the end of the available items...
		while(TextParsed.find(";")==string::npos)
		{
			index++;
			//keep adding new ones
			TextParsed=strtok(NULL,",");
			Armorers[ID].AvailableArmor[index]=atoi(TextParsed.c_str() );
		}

	}

	file_open.close();//close file

	cout<<"File closed\n";
}
